#!/usr/bin/python2.4
# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import service
from google.protobuf import service_reflection
from google.protobuf import descriptor_pb2



_CONTACTGROUP = descriptor.Descriptor(
  name='ContactGroup',
  full_name='ContactGroup',
  filename='generated.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='id', full_name='ContactGroup.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='group_name', full_name='ContactGroup.group_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='member', full_name='ContactGroup.member', index=2,
      number=3, type=11, cpp_type=10, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_ADDRESSBOOK = descriptor.Descriptor(
  name='AddressBook',
  full_name='AddressBook',
  filename='generated.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='group', full_name='AddressBook.group', index=0,
      number=1, type=9, cpp_type=9, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='phone', full_name='AddressBook.phone', index=1,
      number=2, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='birthday', full_name='AddressBook.birthday', index=2,
      number=3, type=11, cpp_type=10, label=1,
      default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='favorite_flag', full_name='AddressBook.favorite_flag', index=3,
      number=4, type=8, cpp_type=7, label=1,
      default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='name', full_name='AddressBook.name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='email', full_name='AddressBook.email', index=5,
      number=6, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_RESPONSE = descriptor.Descriptor(
  name='Response',
  full_name='Response',
  filename='generated.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='success', full_name='Response.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='error_code', full_name='Response.error_code', index=1,
      number=2, type=5, cpp_type=1, label=1,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='error_reason', full_name='Response.error_reason', index=2,
      number=3, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)

import django_types_pb2

_CONTACTGROUP.fields_by_name['member'].message_type = _ADDRESSBOOK
_ADDRESSBOOK.fields_by_name['birthday'].message_type = django_types_pb2._DATE

class ContactGroup(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CONTACTGROUP

class AddressBook(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _ADDRESSBOOK

class Response(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _RESPONSE


_ADDRESSSERVICE = descriptor.ServiceDescriptor(
  name='AddressService',
  full_name='AddressService',
  index=0,
  options=None,
  methods=[
  descriptor.MethodDescriptor(
    name='add_contact',
    full_name='AddressService.add_contact',
    index=0,
    containing_service=None,
    input_type=_ADDRESSBOOK,
    output_type=_RESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='get_group',
    full_name='AddressService.get_group',
    index=1,
    containing_service=None,
    input_type=_CONTACTGROUP,
    output_type=_CONTACTGROUP,
    options=None,
  ),
])

class AddressService(service.Service):
  __metaclass__ = service_reflection.GeneratedServiceType
  DESCRIPTOR = _ADDRESSSERVICE
class AddressService_Stub(AddressService):
  __metaclass__ = service_reflection.GeneratedServiceStubType
  DESCRIPTOR = _ADDRESSSERVICE

