#
#from pbandj.model import ProtocolBuffer
#from django_test.test_app import models
#
#protobuf = ProtocolBuffer()
#
#def generateMessages():
#    one_of_everything = protobuf.genMsg('OneOfEverything',
#                                        models.OneOfEverything)
#    protobuf.addMessage(one_of_everything)
    